<?php

/**
* 
*/
class Account
{
  public  $account_info;
  private $LocalMysqli;
  private $MasterMysqli;

  function __construct() {
    $this->web_db_connect();
  }

  public function load_account_info(){
    $param = '';

    if ( isset( $this->account_info['ID'] ) ){
      $param = $this->account_info['ID'];
      $sql = "SELECT *,NOW() as CurrentTime FROM account_data WHERE ID = ?";
      $stmt = $this->LocalMysqli->prepare($sql);
      $state = $stmt->bind_param( "s", $param );
      $stmt->execute();
      $res=array();
      $this->stmt_bind_assoc($stmt, $res);
      $stmt->fetch();
      if ( $res['i_account'] > 0 ){
        $this->account_info = $res;
        return TRUE;
      }
    }
    if ( isset( $this->account_info['i_account'] ) ){
      $param = $this->account_info['i_account'];
      $sql = "SELECT *,NOW() as CurrentTime FROM account_data WHERE i_account = ?";
      $stmt = $this->LocalMysqli->prepare($sql);
      $state = $stmt->bind_param( "i", $param );
      $stmt->execute();
      $res=array();
      $this->stmt_bind_assoc($stmt, $res);
      $stmt->fetch();
      if ( $res['i_account'] > 0 ){
        $this->account_info = $res;
        return TRUE;
      }
    }
    return FALSE;
    //echo $sql;
  }

  public function set_i_account($i_account){
    $this->account_info['i_account'] = $i_account;
  }

  public function set_account_id($id){
    $this->account_info['ID'] = $id;
  }

  public function update_subscription_on_expiration(){
    if ( strtotime( $this->account_info['CurrentTime'] ) > strtotime($this->account_info['CurrentSubscriptionExpirationDate']) ){
      if ( $this->account_info['CurrentSubscriptionType'] != $this->account_info['NextSubscriptionType'] ){
        try {    
          $sql = "UPDATE account_data SET CurrentSubscriptionType = NextSubscriptionType WHERE i_account = ?";
          $stmt = $this->LocalMysqli->prepare($sql);
          $state = $stmt->bind_param( "i", $this->account_info['i_account'] );
          $stmt->execute();
          $stmt->close();
          $this->load_account_info();
          return true;
        } catch (mysqli_sql_exception $e) {
          throw new Account_Exception("MySQL error ".$this->LocalMysqli->error." <br> Query:<br> $sql", $this->LocalMysqli->errno);
          return false;
        }
      }
    }
  }


  public function update_subscription_plan( $i_subscription_plan, $update_expiration_date = TRUE ){
    include dirname(__FILE__) . '/../etc/' . "SubscriptionPlans.php";
    try {    
      if ( $i_subscription_plan > 0 ){
        if ( $update_expiration_date ){
          $sql = "UPDATE account_data SET CurrentSubscriptionType = ?, NextSubscriptionType = ?, CurrentSubscriptionExpirationDate = DATE_ADD( GREATEST( NOW(), IFNULL( CurrentSubscriptionExpirationDate, NOW() ) ), INTERVAL IFNULL(?,0) MONTH) WHERE i_account = ?";
          $stmt = $this->LocalMysqli->prepare($sql);
          $state = $stmt->bind_param( "iiii", $i_subscription_plan, $i_subscription_plan ,$SubscriptionPlans[$i_subscription_plan]['RenewalPeriodMonths'], $this->account_info['i_account'] );
          $stmt->execute();
          $stmt->close();
        } else {
          $sql = "UPDATE account_data SET CurrentSubscriptionType = ?, NextSubscriptionType = ? WHERE i_account = ?";
          $stmt = $this->LocalMysqli->prepare($sql);
          $state = $stmt->bind_param( "iii", $i_subscription_plan, $i_subscription_plan, $this->account_info['i_account'] );
          $stmt->execute();
          $stmt->close();
        }
      } else {
        $sql = "UPDATE account_data SET NextSubscriptionType = 0 WHERE i_account = ?";
        $stmt = $this->LocalMysqli->prepare($sql);
        $state = $stmt->bind_param( "i", $this->account_info['i_account'] );
        $stmt->execute();
        $stmt->close();
      }
      $this->load_account_info();
      return true;
    } catch (mysqli_sql_exception $e) {
      throw new Account_Exception("MySQL error ".$this->LocalMysqli->error." <br> Query:<br> $sql", $this->LocalMysqli->errno);
      return false;
    }
  }

  public function set_auto_renewal( $flag ){
    $SubscriptionAutoRenewal = 0;
    if ( $flag ) {
      $SubscriptionAutoRenewal = 1;
    }
    try {    
      $sql = "UPDATE account_data SET SubscriptionAutoRenewal = ? WHERE i_account = ?";
      $stmt = $this->LocalMysqli->prepare($sql);
      $state = $stmt->bind_param( "ii", $SubscriptionAutoRenewal, $this->account_info['i_account'] );
      $stmt->execute();
      $stmt->close();
      $this->load_account_info();
      return true;
    } catch (mysqli_sql_exception $e) {
      throw new Account_Exception("MySQL error ".$this->LocalMysqli->error." <br> Query:<br> $sql", $this->LocalMysqli->errno);
      return false;
    }
  }

  

  private function web_db_connect(){
    $filename = '/home/site_lib/connections/local.mysqli.141102_test_web_interface.class.php';
    if(!@include_once( $filename ) ) throw new Account_Exception( "Failed to include ". $filename );
    try {
      $LocalMysql = new LocalMysqli;
      $this->LocalMysqli = $LocalMysql->connection;
      $this->LocalMysqli->set_charset("utf8");
      return $this->LocalMysqli;
    } catch (mysqli_sql_exception $e) {
      throw new Account_Exception("Could not connect to mysql server !".$e);
    }
  }

  private function stmt_bind_assoc (&$stmt, &$out) {
    global $mysqli;
    $data = $stmt->result_metadata();
    $fields = array();
    $out = array();
    $fields[0] = $stmt;
    $count = 1;
    while($field = $data->fetch_field()) {
      $fields[$count] = &$out[$field->name];
      $count++;
    }    
    call_user_func_array(mysqli_stmt_bind_result, $fields);
  }

  function __wakeup(){
    $this->web_db_connect();
  }

  public function _close(){
    $this->LocalMysqli->close();
  }

  function __destruct(){

  }
}

class Account_Exception extends Exception{
  public function __construct($message = null, $code = 0, Exception $previous = null) {
    try {
        mail('nicolas.bondier@switzernet.com.test-google-a.com',
           'Automatic account management error',
           $message . "\n\n" . "Message sent from web server :" . __FILE__);
    } catch(Exception $e) { 
      echo $e."\n";
    }
  }
}

?>