<?php

/**
* 
*/
class Customer
{
  public $customer_info;
  public $update_flag;
  private $LocalMysqli;
  private $MasterMysqli;

  function __construct() {
    $this->web_db_connect();
  }

  public function load_customer_info( $check_update = FALSE ){
    $sql = "SELECT *, NOW() as CurrentTime FROM customer_data WHERE i_customer = ?";
    //echo $sql;
    $stmt = $this->LocalMysqli->prepare($sql);
    $state = $stmt->bind_param( "i", $this->customer_info['i_customer'] );
    $stmt->execute();
    $res=array();
    $this->stmt_bind_assoc($stmt, $res);
    $stmt->fetch();
    $info = false;
    if ( $res['i_customer'] > 0 ){
      $info = true;
    }
    $stmt->close();
    if ( $info ){

      $comp1 = $this->customer_info;
      $this->customer_info = $res;
      $this->get_customer_accounts();
      $comp2 = $this->customer_info;
      unset( $comp1['CurrentTime'] );
      unset( $comp2['CurrentTime'] );

      //print_r($comp1);
      //print_r($comp2);

      if ( $check_update == TRUE && $comp1 != $comp2 ){
        $this->update_flag = TRUE;
      } else {
        $this->update_flag = FALSE;
      }

      return TRUE;
    }
    return FALSE;
  }

  public function authenticate( $user, $password ){
    $sql = "SELECT i_customer FROM customer_data WHERE WebLogin = ? AND WebPassword = ?";
    //echo $sql;
    $stmt = $this->LocalMysqli->prepare($sql);

/* Prepared statement, stage 1: prepare */

    $state = $stmt->bind_param( "ss", $user, $password );


    $stmt->execute();
    $res=array();
    $this->stmt_bind_assoc($stmt, $res);
    $stmt->fetch();
    $stmt->close();
    if ( $res['i_customer'] > 0 ){
      $this->customer_info['i_customer'] = $res['i_customer'];
      return TRUE;
    }
    return FALSE;

  }

  public function charge( $amount, $visible_comment = '', $internal_comment = '' ){
    try {
      $sql = "UPDATE customer_data SET Balance = Balance - ? WHERE i_customer = ?";
      $stmt = $this->LocalMysqli->prepare($sql);
      $state = $stmt->bind_param( "ii", $amount, $this->customer_info['i_customer'] );
      $stmt->execute();
      $stmt->close();
      $this->load_customer_info();
      return true;
    } catch (mysqli_sql_exception $e) {
      throw new Customer_Exception("MySQL error ".$this->LocalMysqli->error." <br> Query:<br> $sql", $this->LocalMysqli->errno);
      return false;
    }
  }

  public function update_payment_method( $payment_method_data ){
    // Get all params.
    try {    
      $sql = "UPDATE customer_data SET AutomaticPaymentMethod = ?, AutomaticPaymentStatus = ? WHERE i_customer = ?";
      $stmt = $this->LocalMysqli->prepare($sql);
      $state = $stmt->bind_param( "ssi", $payment_method_data['AutomaticPaymentMethod'], $payment_method_data['AutomaticPaymentStatus'], $this->customer_info['i_customer'] );
      $stmt->execute();
      $stmt->close();

      if ( $payment_method_data['AutomaticPaymentMethod'] == '' || $payment_method_data['AutomaticPaymentMethod'] == NULL){
        $sql = "UPDATE account_data SET SubscriptionAutoRenewal = 0 WHERE i_customer = ?";
        $stmt = $this->LocalMysqli->prepare($sql);
        $state = $stmt->bind_param( "i", $this->customer_info['i_customer'] );
        $stmt->execute();
        $stmt->close();
      }

      $this->load_customer_info();
      return true;
    } catch (mysqli_sql_exception $e) {
      throw new Customer_Exception("MySQL error ".$this->LocalMysqli->error." <br> Query:<br> $sql", $this->LocalMysqli->errno);
      return false;
    }
  }

  public function get_customer_accounts(){
    
    $accounts = array();

    $sql = "SELECT i_account, ID FROM account_data WHERE i_customer = ?";

    if (!($stmt = $this->LocalMysqli->prepare($sql))) {
        echo "Echec de la préparation : (" . $LocalMysqli->errno . ") " . $LocalMysqli->error;
    }

    $state = $stmt->bind_param( "i", $this->customer_info['i_customer'] );
    $stmt->execute();
    $res=array();
    $this->stmt_bind_assoc($stmt, $res);

    while ( $stmt->fetch() ) {
      $accounts[$res['i_account']] = $res['ID'];
    }

    $this->customer_info['AccountsList'] = $accounts;
    return $accounts;
  }

  private function web_db_connect(){
    $filename = '/home/site_lib/connections/local.mysqli.141102_test_web_interface.class.php';
    if(!@include_once( $filename ) ) throw new Customer_Exception( "Failed to include ". $filename );
    try {
      $LocalMysql = new LocalMysqli;
      $this->LocalMysqli = $LocalMysql->connection;
      $this->LocalMysqli->set_charset("utf8");
      return $this->LocalMysqli;
    } catch (mysqli_sql_exception $e) {
      throw new Customer_Exception("Could not connect to mysql server !".$e);
    }
  }



  private function stmt_bind_assoc (&$stmt, &$out) {
    global $mysqli;
    $data = $stmt->result_metadata();
    $fields = array();
    $out = array();
    $fields[0] = $stmt;
    $count = 1;
    while($field = $data->fetch_field()) {
      $fields[$count] = &$out[$field->name];
      $count++;
    }    
    call_user_func_array(mysqli_stmt_bind_result, $fields);
  }


  public function _close(){
    $this->LocalMysqli->close();
  }


  function __destruct(){

  }
  
  /*
  function __sleep(){
    $this->LocalMysqli->close();

  }
  */
  
  function __wakeup(){
    $this->web_db_connect();
  }
  
}

class Customer_Exception extends Exception{
  public function __construct($message = null, $code = 0, Exception $previous = null) {
    try {
        mail('nicolas.bondier@switzernet.com.test-google-a.com',
           'Automatic account management error',
           $message . "\n\n" . "Message sent from web server :" . __FILE__);
    } catch(Exception $e) { 
      echo $e."\n";
    }
  }
}

?>